require 'spec_helper'

describe UsersController do
  before do
    login
    controller.stub!(:filter_access_to).and_return(true)
  end

  describe "GET 'show'" do
    before do
      User.stub(:find).and_return(@user = stub_model(User))
      get :show, :id => @user.id
    end

    it "should render the 'show' template" do
      response.should render_template("show")
    end

    it "exposes the requested User as @user" do
      assigns[:user].should == @user
    end

    it '@user object should not be a new record' do
      assigns[:user].should_not be_new_record
    end
  end

  describe "GET 'new'" do
    it "should render the 'new' template" do
      get :new
      response.should render_template("new")
    end
  end

  describe "POST 'create'" do
    before :each do
      @user = stub_model(User)
      User.should_receive(:new).and_return(@user)
    end

    it 'should set a @user instance' do
      @user.should_receive(:save).and_return(true)
      post :create, :user => { }
      assigns(:user).should_not be_nil
    end

    it 'should set flash on successful create' do
      @user.should_receive(:save).and_return(true)
      post :create, :user => { }
      flash[:notice].should eq("User successfully created")
    end

    it "should render 'show' template on successful create" do
      @user.should_receive(:save).and_return(true)
      post :create, :user => { }
      response.should redirect_to(user_url(@user))
    end

    it "should render 'new' action on unsuccessful update" do
      @user.should_receive(:save).and_return(false)
      post :create, :user => { }
      response.should render_template("new")
    end
  end

  describe "GET edit" do
    before do
      User.stub(:find).and_return(@user = stub_model(User))
      get :edit, :id => @user.id
    end

    it "exposes the requested User as @user" do
      assigns[:user].should == @user
    end

    it "renders the 'edit' template" do
      response.should render_template("edit")
    end

    it '@user object should not be a new record' do
      assigns[:user].should_not be_new_record
    end
  end

  describe "POST update" do
    before do
      User.stub(:find).and_return(@user = stub_model(User))
    end

    it "should render 'show'" do
      @user.stub!(:update_attributes).and_return(true)
      put :update, :id => @user.id, :user => { }
      response.should redirect_to(user_path(@user))
    end

    it "re-renders the 'edit' template" do
      @user.stub!(:update_attributes).and_return(false)
      put :update, :id => @user.id, :user => { }
      response.should render_template("edit")
    end

    it "should set flash message on successful update" do
      @user.stub!(:update_attributes).and_return(true)
      put :update, :id => @user.id, :user => { }
      flash[:notice].should eq("User successfully updated")
    end
  end

  describe "GET 'index'" do
    it "should render 'index' template" do
      get :index
      response.should render_template("index")
    end

    it "should return with a 200 status code" do
      get :index
      response.status.should == 200
    end

    it 'should assign @users' do
      get :index
      assigns[:users].should_not be_nil
    end
  end
  
  describe "PUT 'update_password'" do    
    it "should render the change_password template if the current password does not match the supplied current password" do
      controller.current_user.stub(:valid_password?).and_return(false)
      put :update_password, :id => controller.current_user.id
      response.should render_template("change_password")
    end

    it "should render the change_password template if the new password does not match the password_confirmation" do
      controller.current_user.stub(:valid_password?).and_return(true)
      controller.current_user.stub(:save).and_return(false)
      put :update_password, :id => controller.current_user.id
      response.should render_template("change_password")
    end

    it "should redirect to the user profile page if the password change is successful" do
      controller.current_user.stub(:valid_password?).and_return(true)
      controller.current_user.stub(:save).and_return(true)
      put :update_password, :id => controller.current_user.id
      response.status.should == 200
    end
  end
end
